>From 79898ee2905b1c8ad7955b015077e0013c300a0c Mon Sep 17 00:00:00 2001
From: Erwan Tulou <brezhoneg1@yahoo.fr>
Date: Sat, 30 May 2009 11:55:52 +0200
Subject: [PATCH] vout reuse : remove width, height and chroma restrictions

---
 src/video_output/video_output.c |  192 ++++++++++++++++++++-------------------
 1 files changed, 98 insertions(+), 94 deletions(-)

diff --git a/src/video_output/video_output.c b/src/video_output/video_output.c
index 8360234..0002c7c 100644
--- a/src/video_output/video_output.c
+++ b/src/video_output/video_output.c
@@ -77,6 +77,7 @@ static void VideoFormatImportRgb( video_format_t *, const picture_heap_t * );
 static void PictureHeapFixRgb( picture_heap_t * );
 
 static void     vout_Destructor   ( vlc_object_t * p_this );
+static void vout_Reset( vout_thread_t* p_vout, video_format_t *p_fmt );
 
 /* Object variables callbacks */
 static int FilterCallback( vlc_object_t *, char const *,
@@ -162,124 +163,47 @@ vout_thread_t *__vout_Request( vlc_object_t *p_this, vout_thread_t *p_vout,
 {
     if( !p_fmt )
     {
-        /* Video output is no longer used.
-         * TODO: support for reusing video outputs with proper _thread-safe_
-         * reference handling. */
+        /* Video output is no longer used. */
         if( p_vout )
             vout_CloseAndRelease( p_vout );
         return NULL;
     }
 
-    /* If a video output was provided, lock it, otherwise look for one. */
-    if( p_vout )
-    {
-        vlc_object_hold( p_vout );
-    }
-
-    /* TODO: find a suitable unused video output */
-
-    /* If we now have a video output, check it has the right properties */
+    /* check that vout can really be reused */
     if( p_vout )
     {
         vlc_mutex_lock( &p_vout->change_lock );
 
-        /* We don't directly check for the "vout-filter" variable for obvious
-         * performance reasons. */
-        if( p_vout->p->b_filter_change )
+        /* restriction to vout reuse : vout-filter */
+        if( p_vout->p->psz_filter_chain && *p_vout->p->psz_filter_chain )
         {
-            char *psz_filter_chain = var_GetString( p_vout, "vout-filter" );
-
-            if( psz_filter_chain && !*psz_filter_chain )
-            {
-                free( psz_filter_chain );
-                psz_filter_chain = NULL;
-            }
-            if( p_vout->p->psz_filter_chain && !*p_vout->p->psz_filter_chain )
-            {
-                free( p_vout->p->psz_filter_chain );
-                p_vout->p->psz_filter_chain = NULL;
-            }
-
-            if( !psz_filter_chain && !p_vout->p->psz_filter_chain )
-            {
-                p_vout->p->b_filter_change = false;
-            }
+            free( p_vout->p->psz_filter_chain );
+            p_vout->p->psz_filter_chain = NULL;
 
-            free( psz_filter_chain );
-        }
-
-        if( p_vout->fmt_render.i_chroma != vlc_fourcc_GetCodec( VIDEO_ES, p_fmt->i_chroma ) ||
-            p_vout->fmt_render.i_width != p_fmt->i_width ||
-            p_vout->fmt_render.i_height != p_fmt->i_height ||
-            p_vout->p->b_filter_change )
-        {
             vlc_mutex_unlock( &p_vout->change_lock );
 
-            /* We are not interested in this format, close this vout */
             vout_CloseAndRelease( p_vout );
-            vlc_object_release( p_vout );
             p_vout = NULL;
         }
         else
         {
-            /* This video output is cool! Hijack it. */
-            if( p_vout->fmt_render.i_aspect != p_fmt->i_aspect )
-            {
-                /* Correct aspect ratio on change
-                 * FIXME factorize this code with other aspect ration related code */
-                unsigned int i_sar_num;
-                unsigned int i_sar_den;
-                unsigned int i_aspect;
-
-                i_aspect = p_fmt->i_aspect;
-                vlc_ureduce( &i_sar_num, &i_sar_den,
-                             p_fmt->i_sar_num, p_fmt->i_sar_den, 50000 );
-#if 0
-                /* What's that, it does not seems to be used correcly everywhere
-                 * beside the previous p_vout->fmt_render.i_aspect != p_fmt->i_aspect
-                 * should be fixed to use it too then */
-                if( p_vout->i_par_num > 0 && p_vout->i_par_den > 0 )
-                {
-                    i_sar_num *= p_vout->i_par_den;
-                    i_sar_den *= p_vout->i_par_num;
-                    i_aspect = i_aspect * p_vout->i_par_den / p_vout->i_par_num;
-                }
-#endif
-
-                if( i_sar_num > 0 && i_sar_den > 0 && i_aspect > 0 )
-                {
-                    p_vout->fmt_in.i_sar_num = i_sar_num;
-                    p_vout->fmt_in.i_sar_den = i_sar_den;
-                    p_vout->fmt_in.i_aspect  = i_aspect;
-
-                    p_vout->fmt_render.i_sar_num = i_sar_num;
-                    p_vout->fmt_render.i_sar_den = i_sar_den;
-                    p_vout->fmt_render.i_aspect  = i_aspect;
-
-                    p_vout->render.i_aspect   = i_aspect;
-
-                    p_vout->i_changes |= VOUT_ASPECT_CHANGE;
-
-                }
-            }
             vlc_mutex_unlock( &p_vout->change_lock );
-
-            vlc_object_release( p_vout );
         }
+    }
 
-        if( p_vout )
-        {
-            msg_Dbg( p_this, "reusing provided vout" );
+    if( p_vout )
+    {
+        msg_Dbg( p_this, "reusing provided vout" );
 
-            spu_Attach( p_vout->p_spu, VLC_OBJECT(p_vout), false );
-            vlc_object_detach( p_vout );
+        vout_Reset( p_vout, p_fmt );
 
-            vlc_object_attach( p_vout, p_this );
-            spu_Attach( p_vout->p_spu, VLC_OBJECT(p_vout), true );
-        }
-    }
+        spu_Attach( p_vout->p_spu, VLC_OBJECT(p_vout), false );
+        vlc_object_detach( p_vout );
 
-    if( !p_vout )
+        vlc_object_attach( p_vout, p_this );
+        spu_Attach( p_vout->p_spu, VLC_OBJECT(p_vout), true );
+    }
+    else
     {
         msg_Dbg( p_this, "no usable vout present, spawning one" );
 
@@ -2029,3 +1953,83 @@ static void DeinterlaceEnable( vout_thread_t *p_vout )
     free( psz_mode );
 }
 
+/*
+ * Reset a previous vout and take new input format into account
+ */
+void vout_Reset( vout_thread_t* p_vout, video_format_t *p_fmt )
+{
+    vlc_mutex_lock( &p_vout->change_lock );
+
+    /* clean up data from previous use of vout */
+    CleanThread( p_vout );
+
+    /* set data from new input */
+    unsigned int i_width = p_fmt->i_width;
+    unsigned int i_height = p_fmt->i_height;
+    vlc_fourcc_t i_chroma = p_fmt->i_chroma;
+    unsigned int i_aspect = p_fmt->i_aspect;
+
+    if( i_width <= 0 || i_height <= 0 || i_aspect <= 0 )
+        return;
+
+    vlc_ureduce( &p_fmt->i_sar_num, &p_fmt->i_sar_den,
+                 p_fmt->i_sar_num, p_fmt->i_sar_den, 50000 );
+    if( p_fmt->i_sar_num <= 0 || p_fmt->i_sar_den <= 0 )
+        return;
+
+    /* Initialize pictures - translation tables and functions
+     * will be initialized later in InitThread */
+    for( int i_index = 0; i_index < 2 * VOUT_MAX_PICTURES + 1; i_index++)
+    {
+        p_vout->p_picture[i_index].i_status = FREE_PICTURE;
+        p_vout->p_picture[i_index].i_type   = EMPTY_PICTURE;
+        p_vout->p_picture[i_index].b_slow   = 0;
+    }
+
+    /* No images in the heap */
+    p_vout->i_heap_size = 0;
+
+    /* Initialize the rendering heap */
+    I_RENDERPICTURES = 0;
+
+    p_vout->fmt_render        = *p_fmt;   /* FIXME palette */
+    p_vout->fmt_in            = *p_fmt;   /* FIXME palette */
+
+    p_vout->render.i_width    = i_width;
+    p_vout->render.i_height   = i_height;
+    p_vout->render.i_chroma   = i_chroma;
+    p_vout->render.i_aspect   = i_aspect;
+
+    p_vout->render.i_rmask    = p_fmt->i_rmask;
+    p_vout->render.i_gmask    = p_fmt->i_gmask;
+    p_vout->render.i_bmask    = p_fmt->i_bmask;
+
+    p_vout->render.i_last_used_pic = -1;
+    p_vout->render.b_allow_modify_pics = 1;
+
+    /* Zero the output heap */
+    I_OUTPUTPICTURES = 0;
+    p_vout->output.i_width    = 0;
+    p_vout->output.i_height   = 0;
+    // p_vout->output.i_chroma   = 0;
+    p_vout->output.i_aspect   = 0;
+
+    //p_vout->output.i_rmask    = 0;
+    //p_vout->output.i_gmask    = 0;
+    //p_vout->output.i_bmask    = 0;
+
+    /* Reset window size */
+    vout_InitWindowSize( p_vout, &p_vout->i_window_width,
+                         &p_vout->i_window_height );
+    vout_Control( p_vout, VOUT_SET_SIZE, p_vout->i_window_width,
+                  p_vout->i_window_height );
+
+    /* re-start vout */
+    InitThread( p_vout );
+
+    vlc_mutex_unlock( &p_vout->change_lock );
+
+    /* reset aspect_ratio */
+    var_SetString( p_vout, "aspect-ratio", "" );
+}
+
-- 
1.5.6.5

